@@ -198,14 +198,21 @@ oparamptr match_instance_param(objinstptr thisinst, char *key)
 /*----------------------------------------------------------------------*/
 /* Find the parameter in the indicated instance by key.  If no such 	*/
 /* instance value exists, return the object (default) parameter.	*/
+/*									*/
+/* find_param() hides instances of expression parameters, returning the	*/
+/* default parameter value.  In cases where the instance value (last	*/
+/* evaluated expression result) is needed, use match_instance_param().	*/
+/* An exception is made when the instance param has type XC_EXPR,	*/
+/* indicating that the instance redefines the entire expression.	*/
 /*----------------------------------------------------------------------*/
 
 oparamptr find_param(objinstptr thisinst, char *key)
 {
-   oparamptr fparam;
+   oparamptr fparam, ops;
    fparam = match_instance_param(thisinst, key);
-   if (fparam == NULL)
-      fparam = match_param(thisinst->thisobject, key);
+   ops = match_param(thisinst->thisobject, key);
+   if ((fparam == NULL) || (ops->type == XC_EXPR) && (fparam->type != XC_EXPR))
+      fparam = ops;
    return fparam;
 }
 
@@ -223,6 +230,16 @@ int get_num_params(objectptr thisobj)
    return nparam;
 }
 
+/*----------------------------------------------------------------------*/
+/* Remove all element parameters from an element			*/
+/*----------------------------------------------------------------------*/
+
+void free_all_eparams(genericptr thiselem)
+{
+   while (thiselem->passed != NULL)
+      free_element_param(thiselem, thiselem->passed);
+}
+
 /*----------------------------------------------------------------------*/
 /* Remove an element parameter (eparam) and free memory associated with	*/
 /* the parameter key.							*/
@@ -429,7 +446,7 @@ int opsubstitute(objectptr thisobj, objinstptr pinst)
    int i, j, k, ival;
    char *key;
    float fval;
-   oparamptr dps, ops;
+   oparamptr dps, ops, cps;
    eparamptr epp;
    XPoint *setpt;
    Boolean needrecalc;	/* for arcs and splines */
@@ -473,7 +490,7 @@ int opsubstitute(objectptr thisobj, objinstptr pinst)
 		  free(promoted);
 		  break;
 	       case XC_EXPR:
-		  if ((promoted = evaluate_expr(dps)) == NULL) continue;
+		  if ((promoted = evaluate_expr(dps, pinst)) == NULL) continue;
 		  if (sscanf(promoted, "%g", &fval) == 1)
 		     ival = (int)(fval + 0.5);
 		  free(promoted);
@@ -1037,9 +1054,13 @@ void resolveparams(objinstptr thisinst)
 	    }
 	    break;
 	 case XC_EXPR:
-	    if (!strcmp(ops->parameter.expr, ips->parameter.expr)) {
-	       free(ips->parameter.expr);
-	       free_instance_param(thisinst, ips);
+	    /* Always retain the last evaluated result, unless	*/
+	    /* the instance redefines the expression.		*/
+	    if (ips->type == XC_EXPR) {
+	       if (!strcmp(ops->parameter.expr, ips->parameter.expr)) {
+	          free(ips->parameter.expr);
+	          free_instance_param(thisinst, ips);
+	       }
 	    }
 	    break;
 	 case XC_INT: case XC_FLOAT:
@@ -1121,11 +1142,17 @@ oparamptr copyparameter(oparamptr cops)
 /* Fill any NULL instance parameters with the values	*/
 /* from the calling instance, or from the instance	*/
 /* object's defaults if destinst = sourceinst.		*/ 
+/*							*/
+/* Expression parameters get special treatment because	*/
+/* the instance value may be holding the last evaluated	*/
+/* expression, not an instance value of the expression.	*/
+/* If so, its type will be XC_STRING or XC_FLOAT, not	*/
+/* XC_EXPR.						*/
 /*------------------------------------------------------*/
 
 void copyparams(objinstptr destinst, objinstptr sourceinst)
 {
-   oparamptr psource, cops, newops;
+   oparamptr psource, cops, newops, ips;
 
    if (sourceinst == NULL) return;
    if (destinst == sourceinst)
@@ -1134,7 +1161,8 @@ void copyparams(objinstptr destinst, objinstptr sourceinst)
       psource = sourceinst->params;
 
    for (cops = psource; cops != NULL; cops = cops->next) {
-      if (match_instance_param(destinst, cops->key) == NULL) {
+      if (((ips = match_instance_param(destinst, cops->key)) == NULL) ||
+		(cops->type == XC_EXPR && ips->type != XC_EXPR)) {
          newops = copyparameter(cops);
 	 newops->next = destinst->params;
 	 destinst->params = newops;
